
; ATMega8515

.include 	"m8515def.inc"

; Version v0.1 27.03.2007 M'Soft Berlin

; programmierte FUSE: SUT0  - Quarz - Slowly Rising Power 

; Definitionen

; Belegungen der Atmel-Ports:

;*****************************************************************

;	PA0		0/D0	SID_D0
;	PA1		1/D1	SID_D1
;	PA2		2/D2	SID_D2
;	PA3		3/D3	SID_D3
;	PA4		4/D4	SID_D4
;	PA5		5/D5	SID_D5
;	PA6		6/D6	SID_D6
;	PA7		7/D7	SID_D7

;	PB0		T0/OC0	SID_E
;	PB1		T1		SID_RES
;	PB2		AIN0
;	PB3		AIN1
;	PB4		SS
;	PB5		MOSI
;	PB6		MISO
;	PB7		SCK

;	PC0		A8		SID_A0						
;	PC1		A9		SID_A1		
;	PC2		A10		SID_A2		
;	PC3		A11		SID_A3	
;	PC4		A12		SID_A4
;	PC5		A13
;	PC6		A14
;	PC7		A15					

;	PD0		RxD		Test-Taste
;	PD1		TxD		Debug-Ausgabe		
;	PD2		INT0
;	PD3		INT1
;	PD4		XCK	
;	PD5		OC1A
;	PD6		WR
;	PD7		RD

;	PE0		INT2	SID_CS
;	PE1		ALE
;	PE2		OC1B	SID_R/W

;*****************************************************************

;--- Allgemeines -----------------------------------------------

.equ    F_CPU           = 16000000
.equ    CYCLES_PER_US   = ((F_CPU+500000)/1000000)

.equ    UART_BAUD_RATE  = 38400
.equ    UART_BAUD_SELECT= (F_CPU/(UART_BAUD_RATE*16))-1

;--- 6510 Flag-Register --------------------------------

.equ	FLAG_I		= 7			; I-Flag des 6510
.equ	FLAG_D		= 6			; Dezimal-Flag 	
.equ	FLAG_B		= 5			; Break-Flag
.equ	FLAG_V		= SREG_V	; 3	 vorerst Lage wie in SREG. Es gibt keine Befehle beim 6510, die direkten Zugriff auf das Status-Register haben?
.equ	FLAG_N		= SREG_N	; 2
.equ	FLAG_Z		= SREG_Z	; 1
.equ	FLAG_C		= SREG_C	; 0

.equ	NZ_FLAGS	= (1<<FLAG_N)|(1<<FLAG_Z)
.equ	NZ_MASK		= ~((1<<FLAG_N)|(1<<FLAG_Z))

.equ	NZC_FLAGS	= (1<<FLAG_N)|(1<<FLAG_Z)|(1<<FLAG_C)
.equ	NZC_MASK	= ~((1<<FLAG_N)|(1<<FLAG_Z)|(1<<FLAG_C))				; Lschmaske 

.equ	NZCV_FLAGS	= (1<<FLAG_N)|(1<<FLAG_Z)|(1<<FLAG_C)|(1<<FLAG_V)
.equ	NZCV_MASK	= ~((1<<FLAG_N)|(1<<FLAG_Z)|(1<<FLAG_C)|(1<<FLAG_V))	; Lschmaske

.equ	BIT_FLAGS	= ~((1<<FLAG_N)|(1<<FLAG_Z)|(1<<FLAG_V))				; Lschmaske

.equ	NMI_VEKTOR	= $F000				; Dummy

.equ	IRQ_MASK	= 0b00000000

;--- SID ---------------------------------------------------------

.equ	SID_R_PORT	= PORTB
.equ	SID_E		= PB0
.equ	SID_RES		= PB1

.equ	SID_CTRL	= PORTE
.equ	SID_CS		= PE0
.equ	SID_RW		= PE2

.equ	SID_DDR		= DDRA
.equ	SID_OUT		= PORTA
.equ	SID_PIN		= PINA

.equ	SID_ADR		= PORTC


.equ	RAM_WR		= PD6
.equ	RAM_RD		= PD7

;**************************** Ram-Reservierungen ******************

;--- Internes Ram -------------------------------------------------

.DSEG
.ORG	$0060
IRQ_VEKTOR:			.byte	2

SID_DATEN:			.byte	64

.ORG	$0100
STACKPAGE:			.byte	256

.ORG	$0400					; C64 Video-Text-Ram, nicht bentigt (hoffentlich...)
ZEROPAGE:			.byte	256

;****************** Port-Zuordnung und Register-Bits ******************

;--- lower Register -----------------------------------------------

.def	MUL_LOW		= r0
.def	MUL_HIGH	= r1

.def	NULL		= r2					; Hilfsregister fr Tabellenoffset


;--- higher Register ----------------------------------------------

.def    TEMP_A      = r16                   ; temporres  8Bit-Register
.def    TEMP_B      = r17                   ; temporres  8Bit-Register
.def    TEMP_C      = r18                   ; temporres  8Bit-Register
.def    TEMP_D      = r19                   ; temporres  8Bit-Register

.def	FLAGS_6510	= r20

.def	AKKU		= r21					; 6510 Akku
.def	X_REG		= r22					; 6510 X-Register	
.def	Y_REG		= r23					; 6510 Y-Register
.def	STATUS		= r24					; 6510 Status-Register

.def	SP			= r25					; 6510 Stackpointer

; X temporres Register fr Adressberchnung X/Y
; Y 6510 Programmzhler
; Z Zeiger Befehlsdekoder usw.


;--- Macros -------------------------------------------------------

.macro  load_p                  		    ; ldt 16Bit @1 nach @0 High und @0 Low
        ldi     @0H,HIGH(@1)
        ldi     @0L,LOW(@1)
.endmacro

.macro  ram_off                  		    ; schaltet ext. Ram aus
		in		STATUS,MCUCR				; #1#
		cbr		STATUS,(1<<SRE)				; #1#	
		out		MCUCR,STATUS 				; #1#
.endmacro

.macro  ram_on                  		    ; schaltet ext. Ram ein
		in		STATUS,MCUCR				; #1#
		sbr		STATUS,(1<<SRE)				; #1#
		out		MCUCR,STATUS				; #1#
.endmacro



