
.include	"Definitionen.inc"

#define DEBUG

;****************************** Verktor-Liste ****************************

.CSEG
                rjmp    Reset


;****************************** Programm-Beginn **********************

;*********************************************************************
;  Initialisierungsroutine
;*********************************************************************

.ORG	INT_VECTORS_SIZE

; Hardware initialisieren

Reset:  ldi     TEMP_A,low(RAMEND)          ; Stack an das interne Ram-Ende
        out     SPL,TEMP_A
        ldi     TEMP_A,high(RAMEND)
        out     SPH,TEMP_A

; Externer Ram ein ***************************************************

		ldi		TEMP_A,(1<<SRE)
		out		MCUCR,TEMP_A

; Port A *************************************************************

        ser     TEMP_A						; Alle PullUp ein bzw. Ausgang auf High
        out     PORTA,TEMP_A
		ser		TEMP_A						; Alles Ausgang, SID-Daten
        out     DDRA,TEMP_A

; Port B *************************************************************

        ser		TEMP_A						; Alle PullUp ein bzw. Ausgang auf High
		out     PORTB,TEMP_A
        out     DDRB,TEMP_A	

; Port C *************************************************************

        ser     TEMP_A						; Alle PullUp ein bzw. Ausgang auf High
        out     PORTC,TEMP_A
		ser		TEMP_A						; Alle Ausgang Adresse SID_A0..SID_A4 						
        out     DDRC,TEMP_A	

; Port D *************************************************************

		ser		TEMP_A						; Alle PullUp ein bzw. Ausgang auf High
        out     PORTD,TEMP_A
        ldi		TEMP_A,(1<<RAM_WR)|(1<<RAM_RD)	; auf H wenn extRam aus
		out     DDRD,TEMP_A 				; Tasten

; Port E *************************************************************

		ser		TEMP_A						; Alle PullUp ein bzw. Ausgang auf High
        out     PORTE,TEMP_A
		ser		TEMP_A						; Alles Ausgang, SID_RES, SID_CS
        out     DDRE,TEMP_A

; UART ***************************************************************

        ldi     TEMP_A,high(UART_BAUD_SELECT)
        out     UBRRH,TEMP_A
        ldi     TEMP_A,low(UART_BAUD_SELECT)
        out     UBRRL,TEMP_A
        ldi     TEMP_A,(1<<TXEN)
        out     UCSRB,TEMP_A
        ldi     TEMP_A,(1<<URSEL)|(1<<UCSZ1)|(1<<UCSZ0)
        out     UCSRC,TEMP_A

; TIMER 8Bit***********************************************************
;	Teiler 8, CTC-Mode, Toggle Pin

		clr		TEMP_A
		out		TCNT0,TEMP_A
		ldi		TEMP_A,(1<<CS00)|(1<<WGM01)|(1<<COM00)
		out		TCCR0,TEMP_A
        ldi     TEMP_A,8					; Timer-Wert
        out     OCR0,TEMP_A		   

		clr		TEMP_A						; kein Timer-IRQ
        out     TIMSK,TEMP_A 

;*********************************************************************				 

; Software initialisieren
		load_p	Z,(2*debug_reset)
		rcall	send_text		
		rcall	send_newline

		ldi		SP,$FF						; Stackpointer auf Ende Stackpage
		rcall	load_sid					; Ladeadresse in PC 6510 (Y)

		load_p	Z,STACKPAGE
		ldi		TEMP_A,$FF

stack_clear_loop:		
		st		Z+,TEMP_A
		tst		ZL
		brne	stack_clear_loop

		push	YL
		push	YH
		mov		XL,YL
		mov		XH,YH
		load_p	Y,512
		rcall	debug_32bit_tab
		pop		YH
		pop		YL
						 
;*********************************************************************
;  Main-Loop
;*********************************************************************
main:

		rcall	sid_test

;		rcall	befehls_decoder	

main_end:
		rjmp	main


;*********************************************************************
;  Spielt eine Tonleiter auf Stimme 1
; 
; 
; Scratch-Reg: -
;*********************************************************************

sid_test:
		load_p	Z,(2*musik)			; Quelle
		load_p	Y,SID_DATEN			; Ziel
		load_p	X,16				; Anzahl
		rcall	copy_rom_ram		; umkopieren

		cbi		SID_R_PORT,SID_RES	; Reset SID auf L
		rcall	zyklus_1			; 5 E-Clock-Zyklen abwarten
		sbi		SID_R_PORT,SID_RES	; #1# Reset SID auf H, E-Clock sollte jetzt H sein

		load_p	Z,SID_DATEN	

		clr		TEMP_A

sid_sync_l_1:
		sbic	PINB,SID_E			; warten bis E-Clock auf L ist
		rjmp	sid_sync_l_1

sid_sync_h_1:
		sbis	PINB,SID_E			; warten bis E-Clock auf H ist
		rjmp	sid_sync_h_1

		out		TCNT0,TEMP_A		; Counter startet jetzt ab 0, sollte also jetzt mit 8 Takten L beginnen

; ab hier ist der SID ready, E beginnt mit H-Phase

		nop							; #1# --> E auf H Ausgleich der 2 RET-Takte des AVR
		nop							; #1#	

									; #2# --> E auf H , 2 Takte vom RET der Subroutine
		nop							; #1#		
		ldi		TEMP_B,$18			; #1# Volumen
		ldi		TEMP_A,$08			; #1#
		rcall	sid_write_byte		; #3# Subroutine beginnt mit H-Phase		

									; #2# --> E auf H , 3 Takte vom RET der Subroutine
		nop							; #1#		
		ldi		TEMP_B,$05			; #1# Sustain/Release
		ldi		TEMP_A,16+9			; #1#
		rcall	sid_write_byte		; #3# Subroutine beginnt mit H-Phase		

									; #2# --> E auf H , 3 Takte vom RET der Subroutine
		nop							; #1#		
		ldi		TEMP_B,$06			; #1# Attack/Decay
		ldi		TEMP_A,(4*16)+4		; #1#
		rcall	sid_write_byte		; #3# Subroutine beginnt mit H-Phase		


		ldi		TEMP_C,8			; Anzahl Tne

sound_loop:
		ldi		TEMP_A,$00			; beim Aufruf aus AVR-Ebene mu E-Clock syncronisiert werden

sid_sync_l_2:
		sbic	PINB,SID_E			; warten bis E-Clock auf L ist
		rjmp	sid_sync_l_2

sid_sync_h_2:
		sbis	PINB,SID_E			; warten bis E-Clock auf H ist
		rjmp	sid_sync_h_2

		out		TCNT0,TEMP_A		; Counter startet jetzt ab 0, sollte also jetzt mit 8 Takten L beginnen

		nop							; #1# --> E auf H Ausgleich der 2 RET-Takte des AVR
		nop							; #1#	

									; #2# --> E auf H , 2 Takte vom RET der Subroutine
		ldi		TEMP_B,$01			; #1# Frequenz H
		ld		TEMP_A,Z+			; #2#
		rcall	sid_write_byte		; #3# Subroutine beginnt mit H-Phase		


									; #2# --> E auf H , 2 Takte vom RET der Subroutine
		ldi		TEMP_B,$00			; #1# Frequenz L
		ld		TEMP_A,Z+			; #2#
		rcall	sid_write_byte		; #3# Subroutine beginnt mit H-Phase		

									; #2# --> E auf H , 2 Takte vom RET der Subroutine
		nop							; #1#
		ldi		TEMP_B,$04			; #1# Ton an
		ldi		TEMP_A,0b00010001	; #1#
		rcall	sid_write_byte		; #3# Subroutine beginnt mit H-Phase		

		ldi		TEMP_A,100

wait_loop:		
		rcall	wait_5ms
		dec		TEMP_A
		brne	wait_loop
		
		dec		TEMP_C
		brne	sound_loop

		ret

;*********************************************************************
;  Schreibt ein Byte zum SID, E-Clock beginnt mit H/L bei Aufruf
; In         : SID-Daten TEMP_A, SID-Adresse TEMP_B
; 
; Scratch-Reg: -
;*********************************************************************

sid_write_byte:

		ram_off						; #3# -> E auf L
		out		SID_ADR,TEMP_B		; #1#
		out		SID_OUT,TEMP_A		; #1# 
		cbi		SID_CTRL,SID_RW		; #1# schreiben ein
		cbi		SID_CTRL,SID_CS		; #1# SID selektiert
		nop							; #1# 
			
		nop							; #1# --> E auf H, SID liest die Daten	
		nop							; #1#	
		nop							; #1#	
		nop							; #1# 
		nop							; #1#	
		nop							; #1#	
		nop							; #1#	
		nop							; #1# 
											
		nop							; #1# --> E auf L	noch 125ns warten 
		sbi		SID_CTRL,SID_RW		; #1# schreiben aus
		sbi		SID_CTRL,SID_CS		; #1# SID deselektiert
		ram_on						; #3#

		ret							; #4# --> E auf H nach dem zweiten Takt !!!	

;*********************************************************************
;  Ende Main-Loop
;*********************************************************************

;*********************************************************************
; 6510 Befehldecoder, Rckkehr nur ber unbekannten OP-Code
; In         : Y zeigt auf Beginn des 6510-Programm, SP = $FF
; 
; Scratch-Reg:
;*********************************************************************

befehls_decoder:
		
#ifdef DEBUG
		rcall	debug_6510
		rcall	get_key
#endif

		nop							; #1# #1#
		nop							; #1# #1#
		nop							; #1# #1#
		nop							; #1# #1#
		nop							; #1# #1#
		in		TEMP_A,TIFR			; #1# #1# Timer-Flags holen (CIA-Timer-Emu)
		andi	TEMP_A,IRQ_MASK		; #1# #1#
		breq	befehls_decoder_ok	; #2# #1#

		ldi		ZL,$02				; 	  #1# Dummy-Opcode $02
		rjmp	befehls_decoder_irq	; 	  #2#

befehls_decoder_ok:
		ld		ZL,Y+				; #2# 6510 Befehl holen

befehls_decoder_irq:
		ldi		ZH,high(befehle)	; #1# #1#
		ijmp						; #2# #2# (+ #2# fr rjmp in Tabelle -> 16 Takte bis zum Befehl)	

unbekannt:
		rcall	debug_6510
		
		ret

;*********************************************************************
; Wrteschleifen fr Befehlsdecoder und SID
; 
; Scratch-Reg:
;*********************************************************************

					; #3# Aufruf rcall
zyklus_1:
		nop			; #1#

warten_15:
		nop			; #1#

warten_14:
		nop			; #1#

warten_13:
		nop			; #1#

warten_12:
		nop			; #1#

warten_11:
		nop			; #1#

warten_10:
		nop			; #1#

warten_9:
		nop			; #1#

warten_8:
		nop			; #1#

warten_7:
		ret			; #4#


;***************************************************************************

.include	"befehle_a.inc"
.include	"befehle_b.inc"
.include	"befehle_c.inc"
.include	"befehle_d.inc"
.include	"befehle_e.inc"
.include	"befehle_i.inc"
.include	"befehle_j.inc"
.include	"befehle_l.inc"
.include	"befehle_n.inc"
.include	"befehle_o.inc"
.include	"befehle_p.inc"
.include	"befehle_r.inc"
.include	"befehle_s.inc"
.include	"befehle_t.inc"
.include	"set_flags.inc"

.include	"subroutinen.inc"
.include	"uart.inc"
.include	"debug.inc"

;***************************************************************************
;  Programm Ende 
;***************************************************************************

.include	"tabellen.inc"
